@@ -4,10 +4,10 @@
 #
 plot.msets=function(x,Layout=c('circular','landscape'),degree=NULL,keep.empty.intersections=TRUE,
 	sort.by=c('set','size','degree','p-value'),min.intersection.size=0,max.intersection.size=Inf,ylim=NULL,
-	log.scale=FALSE,x.pos=c(0.05,0.95),y.pos=c(0.025,0.975),yfrac=0.8,color.scale.pos=c(0.85, 0.9),legend.pos=c(0.85,0.25),legend.col=2,legend.text.cex=1,
+	log.scale=FALSE,yfrac=0.8,margin=NULL,color.scale.pos=c(0.85, 0.9),legend.pos=c(0.85,0.25),legend.col=2,legend.text.cex=1,
 	color.scale.cex=1,color.scale.title=expression(paste(-Log[10],'(',italic(P),')')),color.on='#2EFE64',color.off='#EEEEEE',
-	show.overlap.size=TRUE, show.set.size=TRUE,	overlap.size.cex=0.9, track.area.range=0.3,bar.area.range=0.2,origin=if(sort.by[1]=='size'){c(0.45,0.5)}else{c(0.5,0.5)},pos.size=0.005,
-	new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,...){
+	show.overlap.size=TRUE, show.set.size=TRUE,	overlap.size.cex=0.9, track.area.range=0.3,bar.area.range=0.2,
+	new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,show.elements=FALSE,...){
 #keep.empty.intersections, whether to retain empty intersections in the plot
 	Layout <- match.arg(Layout)
 	if(is.character(color.scale.pos)){
@@ -22,33 +22,41 @@ plot.msets=function(x,Layout=c('circular','landscape'),degree=NULL,keep.empty.in
 		return(plot.msets.circular(x=x,degree=degree,keep.empty.intersections=keep.empty.intersections,sort.by=sort.by,
 		min.intersection.size=min.intersection.size,max.intersection.size=max.intersection.size,ylim=ylim,log.scale=log.scale,color.scale.pos=color.scale.pos,legend.pos=legend.pos,
 		legend.col=legend.col,legend.text.cex=legend.text.cex,color.scale.cex=color.scale.cex,
-		color.scale.title=color.scale.title,color.on=color.on,color.off=color.off,show.overlap.size=show.overlap.size,overlap.size.cex=overlap.size.cex,
-		track.area.range=track.area.range,bar.area.range=bar.area.range,origin=origin,pos.size=pos.size,new.gridPage=new.gridPage,minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue,...))
+		color.scale.title=color.scale.title,color.on=color.on,color.off=color.off,show.overlap.size=show.overlap.size,overlap.size.cex=overlap.size.cex,margin=margin,
+		track.area.range=track.area.range,bar.area.range=bar.area.range,new.gridPage=new.gridPage,minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue,...))
 	}else if(Layout=='landscape'){
 		return(plot.msets.landscape(x=x,degree=degree,keep.empty.intersections=keep.empty.intersections,sort.by=sort.by,
-		min.intersection.size=min.intersection.size,max.intersection.size=max.intersection.size,ylim=ylim,log.scale=log.scale,x.pos=x.pos,y.pos=y.pos,yfrac=yfrac,color.scale.pos=color.scale.pos,color.scale.cex=color.scale.cex,
+		min.intersection.size=min.intersection.size,max.intersection.size=max.intersection.size,ylim=ylim,log.scale=log.scale,yfrac=yfrac,margin=margin,color.scale.pos=color.scale.pos,color.scale.cex=color.scale.cex,
 		color.scale.title=color.scale.title,color.on=color.on,color.off=color.off,show.overlap.size=show.overlap.size,show.set.size=show.set.size,overlap.size.cex=overlap.size.cex,
-		new.gridPage=new.gridPage,minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue,...))
+		new.gridPage=new.gridPage,minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue,show.elements=show.elements,...))
 	}else{
 		stop('Invalid Layout\n')
 	}
 }
 plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by=c('set','size','degree','p-value'),min.intersection.size=0,max.intersection.size=Inf,ylim=NULL,
-	log.scale=FALSE,x.pos=c(0.05,1),y.pos=c(0,1),yfrac=0.8,color.scale.pos=c(0.85, 0.9),color.scale.cex=1,color.scale.title=expression(paste(-Log[10],'(',italic(P),')')),
-	color.on='#2EFE64',color.off='#EEEEEE',show.overlap.size=FALSE,show.set.size=TRUE,overlap.size.cex=0.9,new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,...){
+	log.scale=FALSE,yfrac=0.8,margin=NULL,color.scale.pos=c(0.85, 0.9),color.scale.cex=1,color.scale.title=expression(paste(-Log[10],'(',italic(P),')')),
+	color.on='#2EFE64',color.off='#EEEEEE',show.overlap.size=FALSE,show.set.size=TRUE,overlap.size.cex=0.9,new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,show.elements=FALSE,...){
 	#
 	sort.by = match.arg(sort.by)
 	Args=list(...)
+	enable.debug=ifelse(is.null(Args$enable.debug),FALSE,Args$enable.debug)
 	cex=ifelse(is.null(Args$cex),par('cex'),Args$cex)
-	cex.lab=ifelse(is.null(Args$cex.lab),par('cex.lab'),Args$cex.lab)
+	cex.lab=ifelse(is.null(Args$cex.lab),1.2,Args$cex.lab)
+	ylab=ifelse(is.null(Args$ylab),'Number of elements',Args$ylab)
 	#set color scheme
 	if(is.null(Args$heatmapColor)){
 		heatmapColor = rev(heat.colors(50)[1:50])
 	}else{
 		heatmapColor = Args$heatmapColor
 	}
+	bar.split=Args$bar.split
+	if(!is.null(bar.split)){
+		if(any(bar.split <= ylim[1])) stop('bar.split values are invalid\n')
+		if(any(bar.split >= ylim[2])) stop('bar.split values are invalid\n')
+	}
 	nColors=length(heatmapColor)-1
 	params=getPlotParams(x,nColors,degree=degree,keep.empty.intersections=keep.empty.intersections,sort.by=sort.by,min.intersection.size=min.intersection.size,max.intersection.size=max.intersection.size,ylim=ylim,log.scale=log.scale,Layout='landscape',minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue)
+	if((!is.null(bar.split)) && log.scale==TRUE) bar.split=log(bar.split)
 	ylabel=params$ylabel
 	ylabel0=params$ylabel0
 	ylim=params$ylim
@@ -61,16 +69,37 @@ plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.b
 	nSet=length(x$set.sizes) #number of sets
 	cid=params$cid
 	mlogp=params$mlogp
-
+	#
+	if(show.elements){
+		sumtable=Args$elements.list
+		if(is.null(sumtable)){
+			sumtable=summary(x,degree=degree)$Table
+		}else{
+			if(!any(colnames(sumtable)=='Elements')) stop('elements.list must be a data.frame or matrix with one column named "Elements"\n')
+		}
+	}
 	#start plotting
-	#start a canvas
 	if(new.gridPage) grid.newpage()
-	vp <- viewport(x=0.5,y=y.pos[1]+(y.pos[2]-y.pos[1])/2, width=1, height=y.pos[2]-y.pos[1])
-	pushViewport(vp)
+	#arrange layout
+	if(is.null(margin)) margin=c(0.5,5,1.5,2)+0.1
+	top.vp <- viewport(layout=grid.layout(4, 3, widths=unit(c(margin[2], 1, margin[4]), c("lines", "null", "lines")), heights=unit(c(margin[3], ceiling(yfrac*100),100-ceiling(yfrac*100), margin[1]), c("lines", "null", "null", "lines"))))
+	marginb <- viewport(layout.pos.col = 2, layout.pos.row = 4, name = "marginb")
+	marginl <- viewport(layout.pos.col = 1, layout.pos.row = 2, name = "marginl")
+	margint <- viewport(layout.pos.col = 2, layout.pos.row = 1, name = "margint")
+	marginr <- viewport(layout.pos.col = 3, layout.pos.row = 2, name = "marginr")
+	plot1 <- viewport(layout.pos.col = 2, layout.pos.row = 2, name = "plot1")
+	plot2 <- viewport(layout.pos.col = 2, layout.pos.row = 3, name = "plot2")
+	splot <- vpTree(top.vp, vpList(marginb, marginl, margint, marginr, plot1, plot2))
+	pushViewport(splot)
+	#
+	#plot intersections
+	seekViewport("plot1")
+	if(enable.debug) grid.rect()
+	char.size.h=convertUnit(stringHeight('o'), "npc", "y",valueOnly=TRUE)
 	if(nSet==1){
 		grid.circle(x=0.5, y=0.5, r=0.3)
 		grid.text(otab0[1],0.5,0.5,gp=gpar(cex=cex*2.5))
-		upViewport()
+		upViewport(0)
 		return(invisible())
 	}
 	if(nSet==2){
@@ -80,36 +109,76 @@ plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.b
 		grid.text(otab0['10'],0.3,0.5,gp=gpar(cex=cex*2.5))
 		grid.text(otab0['11'],0.5,0.5,gp=gpar(cex=cex*2.5))
 		grid.text(otab0['01'],0.7,0.5,gp=gpar(cex=cex*2.5))
-		upViewport()
+		upViewport(0)
 		return(invisible())
-	}
-	#sub canvas 1
-	vp1 <- viewport(x=x.pos[1]+(x.pos[2]-x.pos[1])/2, y=0.6, width=(x.pos[2]-x.pos[1]), height=yfrac)
-	pushViewport(vp1)
-	
+	}	
 	yLen=1 #height of y axis
 	w=1/nO
 	h=yLen/(ylim[2]-ylim[1])
-	#plot intersections
+	if((!is.null(bar.split)) && ylim[2] > bar.split[1]) h=yLen/(ylim[2]-ylim[1]-bar.split[2]+bar.split[1])
+	#
+	if(show.elements){
+		elements.rot=45
+		elements.cex=0.9
+		elements.col=1
+		elements.maximum=Inf
+		if(!is.null(Args$elements.rot)) elements.rot=Args$elements.rot
+		if(!is.null(Args$elements.cex)) elements.cex=Args$elements.cex
+		if(!is.null(Args$elements.col)) elements.col=Args$elements.col
+		if(!is.null(Args$elements.maximum)) elements.maximum=Args$elements.maximum
+		ele.text.posy=ifelse(show.overlap.size,char.size.h/3,0)+ifelse(elements.rot!=0,char.size.h,0)
+	}
 	for(i in 1:nO){
 		posx=w*(i-1)+w/2
-		grid.rect(x=posx,y=0.0,width=w*0.8,height=h*otab[i],just=c('center','bottom'),gp=gpar(fill=heatmapColor[cid[i]]))
-		if(show.overlap.size) grid.text(otab0[i],posx,h*otab[i]+as.numeric(convertUnit(stringHeight('o'), "npc", "y"))/3,gp=gpar(cex=overlap.size.cex),vjust=0)
+		if((!is.null(bar.split)) && otab[i] > bar.split[1]){
+			grid.rect(x=posx,y=0.0,width=w*0.8,height=h*bar.split[1],just=c('center','bottom'),gp=gpar(fill=heatmapColor[cid[i]])) #plot bar
+			ytop=h*bar.split[1]
+			if(otab[i] > bar.split[2]){
+				grid.rect(x=posx,y=h*bar.split[1]+char.size.h/3,width=w*0.8,height=h*(otab[i]-bar.split[2]),just=c('center','bottom'),gp=gpar(fill=heatmapColor[cid[i]])) #plot bar
+				ytop=ytop+char.size.h/3+h*(otab[i]-bar.split[2])
+			}
+		}else{
+			grid.rect(x=posx,y=0.0,width=w*0.8,height=h*otab[i],just=c('center','bottom'),gp=gpar(fill=heatmapColor[cid[i]])) #plot bar
+			ytop=h*otab[i]
+		}
+		if(show.overlap.size) grid.text(otab0[i],posx,ytop+char.size.h/3,gp=gpar(cex=overlap.size.cex),vjust=0)
+		if(show.elements){
+			s00=sumtable[names(otab0)[i],'Elements']
+			if(is.null(s00) || is.na(s00)) next
+			elements=strsplit(s00,', ')[[1]]
+			if(length(elements) > elements.maximum) elements=c(elements[1:elements.maximum],'...')
+			for(j1 in 1:length(elements)) grid.text(elements[j1],posx,ytop+(char.size.h+ifelse(elements.rot!=0,char.size.h/3,0))*j1+ele.text.posy,gp=gpar(cex=elements.cex,col=elements.col),rot=elements.rot,vjust=0.5)
+		}
 	}
-	upViewport()
-
-	#y axis
-	vp1 <- viewport(x=0.05+x.pos[1], y=1-yfrac/2, width=0.1, height=yfrac)
-	pushViewport(vp1)
+	#plot y axis
 	atVal=(ylabel-ylim[1])*h
-	grid.yaxis(at=atVal,label=ylabel0,gp=gpar(cex=cex))
-	upViewport()
-#	vp1 <- viewport(x=0.00, y=1-yfrac/2, width=0.1, height=yfrac)
-#	pushViewport(vp1)
-#	grid.text(ylab,1, 0.5, just=c('center'), rot=90, gp=gpar(cex=cex.lab))
-#	upViewport()
-	
-	#color scale
+	if(!is.null(bar.split)){
+		ylabel0=ylabel0[ylabel <= bar.split[1] | ylabel >= bar.split[2]]
+		ylabel1=ylabel[ylabel <= bar.split[1] | ylabel >= bar.split[2]]
+		atVal=(ylabel1-ylim[1])*h
+		for(i in 1:length(ylabel1)){
+			ytop=h*(min(ylabel1[i],bar.split[1])-ylim[1])
+			if(ylabel1[i] > bar.split[2]){
+				ytop=ytop+char.size.h/3+h*(ylabel1[i]-bar.split[2])
+			}
+			atVal[i]=ytop
+		}
+		char.size.w=convertUnit(stringWidth('0'), "npc", "x",valueOnly=TRUE)
+		yaxis1=grid.yaxis(at=atVal,label=ylabel0,gp=gpar(cex=cex))
+		grid.lines(x = c(0),y = c((bar.split[1]-ylim[1])*h,(bar.split[1]-ylim[1])*h+char.size.h/3),gp=gpar(col='white'))
+		grid.lines(x = c(-char.size.w,0),y = (bar.split[1]-ylim[1])*h,gp=gpar(col='grey'))
+		grid.lines(x = c(-char.size.w,0),y = (bar.split[1]-ylim[1])*h+char.size.h/3,gp=gpar(col='grey'))
+	}else{
+		yaxis1=grid.yaxis(at=atVal,label=ylabel0,gp=gpar(cex=cex))
+	}
+	#plot y axis label
+	seekViewport("marginl")
+	if(enable.debug) grid.rect()
+	char.size.w=convertUnit(stringWidth('0'), "npc", "x",valueOnly=TRUE)
+	grid.text(ylab, x = 1-convertX(unit(1,'lines'), 'npc', valueOnly = TRUE)-2*char.size.w * max(nchar(ylabel0)), rot=90, gp=gpar(cex=cex.lab))
+
+	#plot color scale
+	seekViewport("plot1")
 	if((!is.null(x$n)) & (! is.null(mlogp))){
 		if(is.character(color.scale.pos)){
 			if(color.scale.pos == 'topright'){
@@ -134,8 +203,8 @@ plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.b
 		for(i in 2:length(heatmapColor)){
 			grid.polygon(c((i-1)*wc,i*wc,i*wc,(i-1)*wc),c(0.45,0.45,0.7,0.7),gp=gpar(fill=heatmapColor[i],col=NA))
 		}
-		t1=params$minMinusLog10PValue #0 #floor(min(mlogp,na.rm=T));
-		t2=params$maxMinusLog10PValue #ceiling(max(mlogp,na.rm=T))
+		t1=params$minMinusLog10PValue
+		t2=params$maxMinusLog10PValue
 		grid.text(t1,0,0.3,just=c('center','top'),gp=gpar(cex=color.scale.cex))
 		grid.lines(x = c(wc, wc),y = c(0.45, 0.35))
 		grid.text(t2,1,0.3,just=c('center','top'),gp=gpar(cex=color.scale.cex))
@@ -148,9 +217,9 @@ plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.b
 		upViewport()
 	}
 
-	#sub canvas 2, plot matrix
-	vp2 <- viewport(x=x.pos[1]+(x.pos[2]-x.pos[1])/2, y=(1-yfrac)/2, width=(x.pos[2]-x.pos[1]), height=1-yfrac)
-	pushViewport(vp2)
+	#sub canvas 2, plot intersection matrix
+	seekViewport("plot2")
+	if(enable.debug) grid.rect()
 	h=0.9/nSet
 	for(i in 1:nO){
 		posx=w*(i-1)+w/2
@@ -169,15 +238,14 @@ plot.msets.landscape=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.b
 		grid.text(x$set.names[j], 0.1*w, 0.015+(j-0.5)*h,just=c('right','center'),gp=gpar(cex=cex))
 		if(show.set.size==TRUE) grid.text(x$set.sizes[j], 1, 0.015+(j-0.5)*h,just=c('left','center'),gp=gpar(cex=cex))
 	}
-	upViewport()
-	upViewport()
+	upViewport(0)
 	return(invisible())
 }
 plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by=c('set','size','degree','p-value'),min.intersection.size=0,max.intersection.size=Inf,
 	ylim=NULL,log.scale=FALSE,color.scale.pos=c(0.85, 0.9), legend.pos=c(0.85,0.25),legend.col=2,legend.text.cex=1,
 	color.scale.cex=1,color.scale.title=expression(paste(-Log[10],'(',italic(P),')')),color.on='#2EFE64',color.off='#EEEEEE',
-	show.overlap.size=TRUE,overlap.size.cex=0.9,track.area.range=0.3,bar.area.range=0.2,origin=if(sort.by[1]=='size'){c(0.45,0.5)}else{c(0.5,0.5)},
-	pos.size=0.005,new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,...){
+	show.overlap.size=TRUE,overlap.size.cex=0.9,margin=NULL,track.area.range=0.3,bar.area.range=0.2,
+	new.gridPage=TRUE,minMinusLog10PValue=0,maxMinusLog10PValue=NULL,...){
 	#
 	if(is.character(legend.pos)){
 		legend.pos <- legend.pos[1]
@@ -186,9 +254,11 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 	}else{
 		stop('Invalid legend.pos\n')
 	}
+	if(is.null(margin)) margin=c(1,1,1,2)+0.1
 	sort.by = match.arg(sort.by)
 	Args=list(...)
 	cex=ifelse(is.null(Args$cex),0.8,Args$cex)
+	intersection.size.rotate=ifelse(is.null(Args$intersection.size.rotate),TRUE,Args$intersection.size.rotate)
 	#set color scheme
 	if(is.null(Args$heatmapColor)){
 		heatmapColor = rev(heat.colors(50)[1:50])
@@ -217,7 +287,7 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 	if(track.area.range+bar.area.range>0.5) stop('Sum of track.area.range and bar.area.range should not be larger than 0.5\n')
 	width.sets=track.area.range
 	width.intersections=bar.area.range
-	track.offset=ifelse(is.null(Args$phantom.traks),2,as.integer(Args$phantom.traks)) #number of phantom tracks in the middle
+	track.offset=ifelse(is.null(Args$phantom.tracks),2,as.integer(Args$phantom.tracks)) #number of phantom tracks in the middle
 	track.width=width.sets/(nSet+track.offset)
 	bar.width.unit=width.intersections/(ylim[2]-ylim[1])
 	gap.within.track=ifelse(is.null(Args$gap.within.track),0.1,Args$gap.within.track) #ratio of gap width over block width on the same track
@@ -225,25 +295,24 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 
 	#start a canvas
 	if(new.gridPage) grid.newpage()
+	top.vp <- viewport(layout=grid.layout(3, 3, widths=unit(c(margin[2], 1, margin[4]), c("lines", "null", "lines")), heights=unit(c(margin[3], 1, margin[1]), c("lines", "null", "lines"))))
+	margin1 <- viewport(layout.pos.col = 2, layout.pos.row = 3, name = "marginb")
+	margin2 <- viewport(layout.pos.col = 1, layout.pos.row = 2, name = "marginl")
+	margin3 <- viewport(layout.pos.col = 2, layout.pos.row = 1, name = "margint")
+	margin4 <- viewport(layout.pos.col = 3, layout.pos.row = 2, name = "marginr")
+	plot1 <- viewport(layout.pos.col = 2, layout.pos.row = 2, name = "plot1")
+	splot <- vpTree(top.vp, vpList(margin1, margin2, margin3, margin4, plot1))
+	pushViewport(splot)
 
 	#Plot tracks
-	
-	vp1 <- viewport(x=origin[1],y=origin[2],width=0.95, height=0.95)
-	pushViewport(vp1)
+	seekViewport("plot1")
+	origin=c(0.5,0.5)
 	degreeUnit=2*pi/nO
 	degreeStart=(c(1:nO)-1)*degreeUnit
 	degreeEnd=(c(1:nO))*degreeUnit
 	degree.gap=max(degreeUnit*gap.within.track,2*pi/360)
-
+	char.size.h=as.numeric(convertUnit(stringHeight('o'), "npc", "y"))
 	fill.col=rep(c('#dddddd','#999999'),length.out=nSet)
-	#plot tracks
-	#for(i in 1:nSet){
-	#	XY1=sapply(seq(0,2*pi,length.out=360), function(deg) getXY(origin,(i+track.offset-1)*track.width,deg))
-	#	XY2=sapply(seq(0,2*pi,length.out=360), function(deg) getXY(origin,(i+track.offset)*track.width-track.width*gap.between.track,deg))
-	#	pos.x=c(XY1[1,],rev(XY2[1,]))
-	#	pos.y=c(XY1[2,],rev(XY2[2,]))
-	#	grid.polygon(pos.x, pos.y,gp=gpar(col=1))
-	#}
 	#plot overlap (intersection)
 	for(i in 1:nO){
 		which.set=strsplit(names(otab)[i],'')[[1]]=='1'
@@ -262,8 +331,8 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 		grid.polygon(pos.x, pos.y,gp=gpar(fill = heatmapColor[cid[i]],col=1)) #bar height is proportional to the intersection size
 		#text intersection size
 		if(show.overlap.size==TRUE){
-			XY3=sapply(seq(degreeStart[i],degreeEnd[i]-degree.gap,length.out=4), function(deg) getXY(origin,width.sets+pos.size+bar.width.unit*otab[i],deg))
-			grid.text(otab0[i],mean(XY3[1,]),mean(XY3[2,]),rot=degreeStart[i]*radial2deg,just='left',gp=gpar(cex=overlap.size.cex))
+			XY3=sapply(seq(degreeStart[i],degreeEnd[i]-degree.gap,length.out=4), function(deg) getXY(origin,width.sets+bar.width.unit*otab[i]+char.size.h,deg))
+			grid.text(otab0[i],mean(XY3[1,]),mean(XY3[2,]),rot=ifelse(intersection.size.rotate,(degreeStart[i]-pi/2)*radial2deg,0),just='center',gp=gpar(cex=overlap.size.cex))
 		}
 	}
 	#track number (numbering the legends)
@@ -272,8 +341,8 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 		XY2=sapply(seq(degreeStart[1],degreeEnd[1]-degree.gap,length.out=40), function(deg) getXY(origin,(j+track.offset)*track.width,deg))
 		grid.text(j,(XY1[1,1]+XY2[1,1])/2,mean(XY1[2,]),just=c('center'),gp=gpar(cex=cex))
 	}
-	upViewport()
-	#color scale
+
+	#plot color scale
 	if((!is.null(x$n)) & (! is.null(mlogp))){
 		if(is.character(color.scale.pos)){
 			if(color.scale.pos == 'topright'){
@@ -318,7 +387,7 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 		upViewport()
 	}
 
-	#plot legend
+	#plot legend for set names
 	if(is.character(legend.pos)){
 		if(legend.pos == 'topright'){
 			y.vp=0.80
@@ -350,7 +419,7 @@ plot.msets.circular=function(x,degree=NULL,keep.empty.intersections=TRUE,sort.by
 		vgap=ifelse(is.null(Args$legend.vgap),0.1*legend.text.cex,Args$legend.vgap),hgap=ifelse(is.null(Args$legend.hgap),0.2*legend.text.cex,Args$legend.hgap),
 		gp=gpar(cex=legend.text.cex), draw = FALSE), height = unit(1, "null"),side = ifelse(x.vp>0.5,"right",'left'))
 	grid.draw(Legend)
-	upViewport()
+	upViewport(0)
 	return(invisible())
 }
 getXY=function(origin,radius,degree){
@@ -395,7 +464,7 @@ getPlotParams=function(x,nColors=50,degree=NULL,keep.empty.intersections=TRUE,so
 	if(is.null(ylim)){
 		ylabel=axisTicks(c(0,max(otab0,na.rm=T)),log=FALSE) #for landscape layout
 		ylim=c(0,max(otab,na.rm=T))
-		if(Layout=='landscape' & (ylim[2]-max(ylabel))/(ylabel[2]-ylabel[1])>0.5) {ylabel=c(ylabel,ylabel[length(ylabel)]+ylabel[2]-ylabel[1]);ylim[2]=ylabel[length(ylabel)]}
+		if(Layout=='landscape' && (ylim[2]-max(ylabel))/(ylabel[2]-ylabel[1])>0.5) {ylabel=c(ylabel,ylabel[length(ylabel)]+ylabel[2]-ylabel[1]);ylim[2]=ylabel[length(ylabel)]}
 	}else{
 		ylabel=axisTicks(ylim[1:2],log=FALSE)
 	}
@@ -410,13 +479,15 @@ getPlotParams=function(x,nColors=50,degree=NULL,keep.empty.intersections=TRUE,so
 	nO=length(otab)
 	cid=rep(1,nO) #color gradient id
 	mlogp=NULL
-	if((!is.null(x$n)) & nO>0){
+	if((!is.null(x$n)) && nO>0){
 		mlogp=-log(x$P.value,base=10)
 		mlogp[is.na(mlogp)]=1.0e-10
 		mlogp=mlogp[otab.order]
 		mlogp=mlogp[otab.kept]
 		mlogp[mlogp == Inf] = max(320,mlogp[mlogp < Inf],na.rm=TRUE)
 		if(is.null(maxMinusLog10PValue)) maxMinusLog10PValue=ceiling(max(c(mlogp,1e-10),na.rm=TRUE))
+		maxMinusLog10PValue=ceiling(maxMinusLog10PValue)
+		minMinusLog10PValue=floor(minMinusLog10PValue)
 		mlogp[mlogp>maxMinusLog10PValue]=maxMinusLog10PValue
 		cid=ceiling(nColors*(mlogp-minMinusLog10PValue)/(maxMinusLog10PValue-minMinusLog10PValue))
 		cid[cid<1]=1
@@ -424,7 +495,3 @@ getPlotParams=function(x,nColors=50,degree=NULL,keep.empty.intersections=TRUE,so
 	}
 	return(list(otab=otab,otab0=otab0,cid=cid,mlogp=mlogp,ylim=ylim,ylabel=ylabel,ylabel0=ylabel0,minMinusLog10PValue=minMinusLog10PValue,maxMinusLog10PValue=maxMinusLog10PValue))
 }
-axisLen=function(Max,n.ticks=5){
-	u=ceiling(Max/n.ticks)
-	u*n.ticks
-}
